/* ----------------------------------------------------------------------------
 * Copyright (c) Huawei Technologies Co., Ltd. 2021-2021. All rights reserved.
 * Description: Memory Test Case
 * Author: Huawei LiteOS Team
 * Create: 2021-06-02
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission. THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ---------------------------------------------------------------------------
 */

#include "it_los_mem.h"

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif /* __cplusplus */
#endif /* __cplusplus */

static UINT32 TestCase(VOID)
{
    UINT32 ret, size, boundary;
    UINT32 freeSize[2] = {0};  // Array size: 2
    UINT32 size1 = 0x2000;
    UINT32 t = 5;  // 5 for generated unaligned size.
    VOID *p;

    // unaligned size
    size = (TEST_MEM_SIZE / size1) * t + 1;
    boundary = 0x1000;

    MEM_START();

    MEM_INIT();
    freeSize[0] = MemGetFreeSize(g_pool);

    p = LOS_MemAllocAlign(g_pool, size, boundary);
    ICUNIT_GOTO_NOT_EQUAL(p, NULL, 1, EXIT);

    ret = LOS_MemFree(g_pool, p);
    ICUNIT_GOTO_EQUAL(ret, LOS_OK, ret, EXIT);

    freeSize[1] = MemGetFreeSize(g_pool);
    ICUNIT_GOTO_EQUAL(freeSize[0], freeSize[1], 0, EXIT);
EXIT:
    MEM_FREE();
    MEM_END();
    return LOS_OK;
}

/**
 * @ingroup TEST_MEM
 * @par TestCase_Number
 * ItLosMem002
 * @par TestCase_TestCase_Type
 * Function test
 * @brief After alloc memory by LOS_MemAllocAlign and LOS_MemFree memory,The free memory in pool will not decrease.
 * @par TestCase_Pretreatment_Condition
 * NA.
 * @par TestCase_Test_Steps
 * step1: record free memory and alloc memory by LOS_MemAllocAlign,then free it, check free memory is not decrease.
 * @par TestCase_Expected_Result
 * 1.The free memory in pool will not decrease at the end.
 * @par TestCase_Level
 * Level 0
 * @par TestCase_Automated
 * true
 * @par TestCase_Remark
 * null
 */

VOID ItLosMem002(VOID)
{
    TEST_ADD_CASE("ItLosMem002", TestCase, TEST_LOS, TEST_MEM, TEST_LEVEL0, TEST_FUNCTION);
}

#ifdef __cplusplus
#if __cplusplus
}
#endif /* __cplusplus */
#endif /* __cplusplus */
